/**
 * Copyright (C) 2003-2005 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package sync4j.framework.server.session;

import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;

import sync4j.framework.core.*;
import sync4j.framework.engine.dm.DeviceDMState;
import sync4j.framework.security.SecurityConstants;
import sync4j.framework.security.Sync4jPrincipal;
import sync4j.framework.server.SyncTimestamp;
import sync4j.framework.server.Sync4jDevice;
import sync4j.framework.protocol.ProtocolUtil;
import sync4j.framework.core.dm.ddf.DevInfo;


/**
 * This class keeps state information of a device management session.
 *
 * @author Stefano Fornari @ funambol.com
 * @version $Id: ManagementState.java,v 1.1 2005/05/16 17:32:56 nichele Exp $
 */
public class ManagementState
implements SecurityConstants {

    // --------------------------------------------------------------- Constants

    // ------------------------------------------------------------ Private data

    /**
     * Cache AbstractCommands (Alert, Put, Get)
     */
    private ArrayList clientCommands;

    // ------------------------------------------------------------- Public data

    /**
     * Cache the devInf replace command
     */
    public Replace devInfReplaceCommand;

    /**
     * Cache the device info
     */
    public DevInf deviceInfo;

    /**
     * Cache the Sync4jDevice object (it contains Sync4j information and nonces
     */
    public Sync4jDevice device;

    /**
     * Cache the device DM state
     */
    public DeviceDMState dmstate;

    /**
     * The management session id
     */
    public String sessionId;

    /**
     * Authenticated credential. If null no credential has authenticated.
     */
    public Cred loggedCredential;

    /**
     * Authenticated principal. If null, no principal has authenticated
     */
    public Sync4jPrincipal loggedPrincipal;

    /**
     * The Last message Id from the client
     */
    public String lastMsgIdFromClient;

    /**
     * Timestamps of the last management session
     */
    public SyncTimestamp lastTimestamp;

    /**
     * Timestamps of the current management session
     */
    public SyncTimestamp nextTimestamp;

    /**
     * Authentication state. One of the following values:
     * <ul>
     *   <li>AUTH_UNAUTHENTICATED</li>
     *   <li>AUTH_MISSING_CREDENTIALS</li>
     *   <li>AUTH_INVALID_CREDENTIALS</li>
     *   <li>AUTH_AUTHENTICATED</li>
     * </ul>
     */
    public int authenticationState;

    /**
     * Server authentication state. One of the following values:
     * <ul>
     *   <li>AUTH_UNAUTHENTICATED</li>
     *   <li>AUTH_RETRY_1</li>
     *   <li>AUTH_RETRY_2</li>
     * </ul>
     */
    public int serverAuthenticationState;

    /**
     * Management session type (@see AlertCode)
     */
    public int type;

    /**
     * Has the session been started ?
     */
    public boolean started;


    /**
     * Cache the SyncML Protocol version used by device
     */
    public String syncMLVerProto;


    /**
     * Cache the maximum message size
     */
    private long maxMsgSize = 0;
    public void setMaxMsgSize(long value) {
        maxMsgSize = value;
    }

    public long getMaxMsgSize() {
        return maxMsgSize;
    }

    /**
     * Cache the maximum object size
     */
    private long maxObjSize = 0;
    public long getMaxObjSize() {
        return this.maxObjSize;
    }
    public void setMaxObjSize(long maxObjSize) {
        this.maxObjSize = maxObjSize;
    }


    /**
     * Cache overhead for SyncHdr object
     */
    private long overheadHdr = 0;
    public long getOverheadHdr() {
        return this.overheadHdr;
    }
    public void setOverheadHdr(long overheadHdr) {
        this.overheadHdr = overheadHdr;
    }


    /**
     * Cache the Status command presents in the response message
     */
    private LinkedList listStatusCmdOut = new LinkedList();

    public void addStatusCmdOut(List statusList) {
        listStatusCmdOut.addAll(statusList);
    }

    public LinkedList getStatusCmdOut() {
        return listStatusCmdOut;
    }

    public void removeStatusCmdOut(List statusCommand) {
        listStatusCmdOut.removeAll(statusCommand);
    }


    /**
     * Cache the Alert command presents in the response message
     */
    private LinkedList listAlertCmdOut = new LinkedList();

    public void addAlertCmdOut(List alertList) {
        listAlertCmdOut.addAll(alertList);
    }

    public LinkedList getAlertCmdOut() {
        return listAlertCmdOut;
    }

    public void removeAlertCmdOut(List alertList) {
        listAlertCmdOut.removeAll(alertList);
    }


    /**
     * Cache cmd splitted in more message (Large Object)
     */
    private AbstractCommand splittedCommand = null;

    public AbstractCommand getSplittedCommand() {
        return splittedCommand;
    }

    public void setSplittedCommand(AbstractCommand splittedCommand) {
        this.splittedCommand = splittedCommand;
    }


    /**
     * Cache received large object
     */
    private String receivedLargeObject = null;

    public String getReceivedLargeObject() {
        return receivedLargeObject;
    }

    public void setReceivedLargeObject(String receivedLargeObject) {
        this.receivedLargeObject = receivedLargeObject;
    }

    /**
     * Cache size of received large object
     */
    private Long sizeOfReceivedLargeObject = null;

    public void setSizeOfReceivedLargeObject(Long size) {
        this.sizeOfReceivedLargeObject = size;
    }

    public Long getSizeOfReceivedLargeObject() {
        return sizeOfReceivedLargeObject;
    }


    /**
     * Cache data to send in next messages (Large Object)
     */
    private String nextDataToSend = null;

    public String getNextDataToSend() {
        return nextDataToSend;
    }

    public void setNextDataToSend(String nextDataToSend) {
        this.nextDataToSend = nextDataToSend;
    }



    /**
     * Cache the AbstractCommand presents in the response message
     */
    private LinkedList listCmdOut = new LinkedList();

    public void addCmdOut(List cmdList) {
        listCmdOut.addAll(cmdList);
    }

    public void addCmdOut(int index, List cmdList) {
         listCmdOut.addAll(index, cmdList);
     }

    public LinkedList getCmdOut() {
        return listCmdOut;
    }

    public void removeCmdOut(List abstractCommandsList) {
        listCmdOut.removeAll(abstractCommandsList);
    }

    public void removeCmdOut(AbstractCommand abstractCommand) {
        listCmdOut.remove(abstractCommand);
    }


    /**
     * Cache overhead of Status for SyncHdr
     */
    private long sizeStatusSyncHdr = 0;
    public long getSizeStatusSyncHdr() {
        return this.sizeStatusSyncHdr;
    }

    public void setSizeStatusSyncHdr(long sizeStatusSyncHdr) {
         this.sizeStatusSyncHdr = sizeStatusSyncHdr;
     }

     /**
      * Cache Status for SyncHdr
      */
     private Status statusSyncHdr;
     public void setStatusSyncHdr(Status statusSyncHdr) {
         this.statusSyncHdr = statusSyncHdr;
     }

     public Status getStatusSyncHdr() {
         return this.statusSyncHdr;
     }


    // ------------------------------------------------------------ Constructors

    /**
     * Creates a new management state object associated to the given session id
     *
     * @param sessionId the sessionId
     */
    public ManagementState(String sessionId) {
        this();
        this.sessionId = sessionId;
    }

    /**
     * Creates a new management state object with empty state and not associated
     * to any session id.
     */
    public ManagementState() {
        reset();
    }



    // ---------------------------------------------------------- Public Methods

    public void addClientCommands(AbstractCommand[] commands) {
        for (int i=0; ((commands != null) && (i<commands.length)); i++) {
            if (ProtocolUtil.hasLargeObject(commands[i])) {
                // commands with large object don't come cached
                continue;
            }
            clientCommands.add(commands[i]);
        }
    }

    public AbstractCommand[] getClientCommands() {
        return (AbstractCommand[])clientCommands.toArray(new AbstractCommand[clientCommands.size()]);
    }

    public void clearClientCommands() {
        clientCommands.clear();
    }


    /**
     * Sets sessionId
     *
     * @param sessionId the management session id
     */
    protected void setSessionId(String sessionId) {
        this.sessionId = sessionId;
    }

    /**
     * Returns the management session id
     *
     * @return the management session id
     */
    public String getSessionId() {
        return sessionId;
    }

    /**
     * Reset this object to an initial state
     */
    public void reset() {
        loggedCredential    = null;
        loggedPrincipal     = null;
        lastMsgIdFromClient = null;
        sessionId           = null;
        lastTimestamp       = null;
        nextTimestamp       = null;

        started             = false;

        type                = AlertCode.UNKNOWN;

        authenticationState = AUTH_UNAUTHENTICATED;

        serverAuthenticationState = AUTH_UNAUTHENTICATED;

        clientCommands = new ArrayList();
    }
}